// Filename:	sysexec
// Contents:	the object that encapsulates unusual system operations
// Author:	Greg Shaw
// Created:	5/18/94

/*
This file is free software; you can redistribute it and/or modify it
under the terms of the GNU General Public License as published by the
Free Software Foundation; either version 2, or (at your option) any
later version.

In addition to the permissions in the GNU General Public License, the
Free Software Foundation gives you unlimited permission to link the
compiled version of this file with other programs, and to distribute
those programs without any restriction coming from the use of this
file.  (The General Public License restrictions do apply in other
respects; for example, they cover modification of the file, and
distribution when not linked into another program.)

This file is distributed in the hope that it will be useful, but
WITHOUT ANY WARRANTY; without even the implied warranty of
MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
General Public License for more details.

You should have received a copy of the GNU General Public License
along with this program; see the file COPYING.  If not, write to
the Free Software Foundation, 675 Mass Ave, Cambridge, MA 02139, USA.  */

#ifndef _SYSEXEC_C_
#define _SYSEXEC_C_

#include "bbshdr.h"

// Method:	constructor
// Function:	initialize the object
// Author:	Greg Shaw
// Created:	5/20/94

sysexec::sysexec()
{
	running_pid = 0;	// nothing running
}

// Method:	destructor
// Function:	clean up the object
// Author:	Greg Shaw
// Created:	5/20/94

sysexec::~sysexec()
{
	if (running_pid > 0)	// child running?
	{
		kill(running_pid,SIGHUP);
	}
}

// Method:	sysint
// Function:	serve as a system() command that correctly handles SIGHUP
// Input:	command - the command to execute
// Output:	(command is executed)
// Author:	Greg Shaw
// Created:	5/18/94

int sysexec::sysint(char *command)
{
	int		status;
	sigset_t	mask;	// old signal mask

	if (command == NULL)	// null command?
		return(1);	// error
	
	sigprocmask(SIG_BLOCK, NULL, &mask); // get current blocked sigs mask
	sigaddset(&mask, SIGCHLD);  // add sigchld and sigquit
	sigaddset(&mask, SIGQUIT);
	sigprocmask(SIG_SETMASK, &mask, NULL);	 // set new mask
        if (!(running_pid = fork())) {


                setgid(getgid());
                setuid(getuid());

                execl("/bin/sh", "sh", "-rc", command, (char *) 0);
		ap_log("system: exec failure");
                exit(127);
        }
        if (running_pid == -1) {
		ap_log("system: exec failure");
	 	status = -1;
	}
	else
	{
//		sigprocmask(SIG_BLOCK, NULL, &mask); // get current mask
//		sigaddset(&mask, SIGINT);
//		sigprocmask(SIG_SETMASK, &mask, NULL);	 // set new mask
		while (waitpid(running_pid, &status, 0) < 0)
			if (errno != EINTR) 
			{
				status = -1;
				break;
			}
	}
	running_pid = 0;
//	sigprocmask(SIG_BLOCK, NULL, &mask); // get current mask
//	sigdelset(&mask, SIGINT);
//	sigdelset(&mask, SIGCHLD);
//	sigdelset(&mask, SIGQUIT);
//	sigprocmask(SIG_SETMASK, &mask, NULL);	 // set new mask
	return(status);
	
}


#endif // _SYSEXEC_C_
